%% Calculates Fourier spectra of all records from data/ directory, 
%% saves them in structure 'spec' in the same .mat files,
%% and displays in Figure 21
%%
%% General processing parameters need to be provided in structure 'procdata':
%%	procdata.time_range = (for example) [ -10 0 35 40 ] 	
%%		Analysis time window:
%%		[ left end of taper, start of record, end of record, right end of taper ]
%% Parameters:
%%	ffid - FFID number for which to create spectra
%%
%% Usage examples:
%%	plot_spectra(150)

function create_spectra(ffid)

global procdata

  % load record

  [ rec, ~, ~, dataf ] = record_ffid(ffid);

  %% save parameters of time sampling in 'spectr'

  spec.dt = rec.times(2)-rec.times(1);	% nominal sampling interval 
					% for spectral calculations

  spec.time_range = procdata.time_range;

  %% extract time segment for spectral analysis and taper its ends:

  ind = find(rec.times > procdata.time_range(1) & rec.times < procdata.time_range(4));	
					% record rows we need to keep

  d = rec.data(ind,:);		% extracted segment
  t = rec.times(ind,:);

  d = d.*taper_cos(t, procdata.time_range(2),procdata.time_range(1)) ...
	.* taper_cos(t, procdata.time_range(3),procdata.time_range(4));

  %% 1) calculate complex spectra of field records (excluding the times)

  [ spec.data, spec.fi, spec.fN ] = spectrum(d, spec.dt);
 
  %% form column of output frequencies (from 0 to Nyquist)

  f = transpose(0:(spec.fN-1))*spec.fi;

  %% truncate the spectra to below Nyquist frequency

  spec.data((spec.fN+1):end,:) = [];

  %% 2) plot logarithms of average amplitude spectra within three offset ranges

  aoffs = abs(rec.offset);

  maxdist = max(aoffs);		% maximum distance

  figure(21)	% use figure numbers that do not overlap with waveform plots,
		% so that you can run various scripts simultaneously
  clf
  subplot(1,3,1:2)		% plot waveforms

  plot_ffid1(rec,0.6,spec.time_range)

  subplot(1,3,3)		% plot spectra
  hold on

  linestyle = [ '-b'; '-r'; '-g'; '-m'; 'k' ];

  leg = [];	% plot legend

	% plot by channel ranges

  for i = 1:size(procdata.chan_range,1)	
  	d1 = procdata.chan_range(i,:);

  	ind1 = find(rec.chan >= d1(1) & rec.chan <= d1(2) );

  	if ~isempty(ind1)
		plot(f,10*log10(mean(abs(spec.data(:,ind1).^2),2)),linestyle(rem(i,5)+1,:))

		if isempty(leg)
  			leg = sprintf("chan %d - %d", d1(1), d1(2) );
		else
  			leg = [ leg; sprintf("chan %d - %d", d1(1), d1(2) ) ];
		end
  	end
  end

	% plot by distance ranges

  for i = 1:size(procdata.dist_range,1)	
  	d1 = procdata.dist_range(i,:)*maxdist;

  	ind1 = find(aoffs >= d1(1) & aoffs < d1(2) );

  	if ~isempty(ind1)
		plot(f,10*log10(mean(abs(spec.data(:,ind1).^2),2)),linestyle(rem(i,5)+1,:))

		if isempty(leg)
  			leg = sprintf("%1.1f - %1.1f m", d1(1), d1(2) );
		else
  			leg = [ leg; sprintf("%1.1f - %1.1f m", d1(1), d1(2) ) ];
		end
  	end
  end



  legend(leg)

  xlabel('Frequency (kHz)')
  ylabel('Amplitude (dB)')
  grid on
  title(sprintf('Amplitude spectra in FFID %d',rec.ffid))

  % save data into the same file in binary Matlab format
  
  if inOctave()
  	save(dataf,'-mat7-binary','rec','spec');
  else
	save(dataf,'rec','spec');
  end
end

