%% Loads one raw shot record by FFID number from rdirectory 'mat/',
%% applies editing, and saves in directory 'data/'.
%%
%% Editing includes:
%%	setting 'kill_list' flags
%%	picking approximate time of the source
%%	shifting time values to equal 0 at the picked source
%%	reducing the time range  
%%
%% Editing parameters can be provided in global structure 'procdata'
%% and also given in the arguments of this function.
%%
%% The edited record is saved in directory 'data/'
%% ====================================================================
%%
%% Parameters:
%%	ffid - ffid number to plot and edit
%%	kill_list - list of channels to mark as killed, in addition to those given in 'procdata.kill_list'
%%	time_start - start (source) time value in the record. If this argument is empty ([]),
%%		time_start is auto-picked  by linear regression of the times of the largest positive 
%%		values within 10 channels near the source
%%	time_range - time window to extract into the output records, relative to the time_start.
%%		For example, time_range  = [-100, 1000] means that the output time values 
%%		will start at -100 ms and end at 1000 ms.
%%		If this argument is empty ([]), procdata.time_range is used.
%%		If procdata.time_range is empty, the entire time range of raw records is output.
%% Returns:
%%	rec - edited record structure
%%
%% Usage examples:
%%	rec = edit_ffid(150);

function rec = edit_ffid(ffid,kill_list=[], time_start=[], time_range=[])

global procdata

  %% load record

  rec = record_ffid(ffid,'raw');

  if isempty(rec)
	disp(sprintf("FFID %d does not exist\n",ffid))
	return
  end

  %% drop unused channels

  rec.data(:,procdata.drop_chan) = [];
  rec.chan(procdata.drop_chan) = [];
  rec.rec_stn(procdata.drop_chan) = [];
  rec.offset(procdata.drop_chan) = [];

  %% combine killed records with those listed in procdata

  rec.kill = [ kill_list, procdata.kill ];

  %% cancel kill for channels listed in 'unkill' list

  rec.kill(find(ismember(rec.kill,procdata.unkill))) = [];
	
  %% completely drop killed records if requested

  if procdata.drop_killed 
  	ind = find(ismember(rec.chan,rec.kill));
	
	rec.data(:,ind) = [];
	rec.chan(:,ind) = [];
  	rec.rec_stn(:,ind) = [];
  	rec.offset(:,ind) = [];
  end

  %% override source and receiver station numbers and inline coordinates

  if exist('survey_rec')
  	[rec.rec_stn, rec.rec_x ] = survey_rec(rec.ffid,rec.chan);
  else
	rec.rec_x = rec.line(3)+ (rec.rec_stn - rec.line(2))*rec.line(4);
  end

  if exist('survey_src')
  	[rec.src_stn, rec.src_x, x, rec.src_offset ] = survey_src(rec.ffid);
	rec.src_skid = x/rec.line(4);	% measure src_skid in station spacings
  else
	rec.src_skid = 0.0;			% rec.src_x will be set later
	rec.src_offset = procdata.src_offset;
  end

  %% correct the source coordinate if needed

  if abs(procdata.src_chan_shift) > 1.0e-5
  	j = floor(procdata.src_chan_shift);

  	rec.src_stn = rec.src_stn + j;

  	rec.src_skid = procdata.src_chan_shift - j;
  end

  %% determine the inline source coordinates

  x = rec.src_stn+rec.src_skid;		% fractional source station number (temporary variable)

  rec.src_x = interp1(rec.rec_stn,rec.rec_x,x,'extrap');

  %% determine the 3-D coordinates

  if exist("survey_coords")
	xyz = survey_coords(rec.line,rec.rec_stn);

  	rec.rec_xyz = xyz(:,2:4);

	xyz = survey_coords(rec.line,x);

  	rec.src_xyz = xyz(:,2:4);
	rec.src_xyz(2) = rec.src_xyz(2) + rec.src_offset;	% cross-line offset

  else
	rec.rec_xyz = [ reshape(rec.rec_x,[],1); zeros(length(rec.rec_x),2) ];

  	rec.src_xyz = [ rec.src_x, 0.0, 0.0 ];
  end

  %% recalculate offsets

  if procdata.offsets_xyz		% use 3-D offsets

	x = rec.rec_x - rec.src_x;

	rec.offset = reshape(sqrt(sum((rec.rec_xyz - rec.src_xyz).^2,2)),1,[]).*sign(x);	% save as a row

  else			% use inline offset (maybe if geometry is inaccurate) 

	x = rec.rec_x - rec.src_x;

  	rec.offset = sqrt(x.^2 + rec.src_offset^2).*sign(x);
  end

  % filter records for plotting and picking

  data = bp_filter( rec.data, rec.dt, procdata.filter_bp, procdata.filter_notch );

  %% pick first arrival times and measure their amplitudes

  rec.picks_fb = pick_fbreaks(rec,data);

  % data = stalta(data,rec.dt);		% test plotting STA/LTA ratio instead of data

  %% derive travel-time model for near traces

  if ~isempty(rec.picks_fb)

	% select up to 10 traces with largest amplitudes

  	[ ~, ind ] = sort(rec.picks_fb(:,4),'descend');

	if length(ind) > procdata.pick_fb.near_trc
		ind((procdata.pick_fb.near_trc+1):end) = [];
	end
  
	ind_near = rec.picks_fb(ind,1);

  	o = abs(rec.offset(ind_near));		% unsigned offsets
  	t = transpose(rec.picks_fb(ind,2));	% times of peaks

  	rec.tt_near = polyfit(o,t,1);		% linear fitting
  
	ind_near = sort(ind_near);		% make sure indices are in sequence
  else
	rec.tt_near = [];
	ind_near = [];
  end

  %% estimate time of the source from near traces

  if isempty(time_start) && ~isempty(rec.tt_near)

  	time_start = polyval(rec.tt_near,0.0);	% time at zero offset
  end

  rec.time_start = time_start;

  %% subtract source time

  rec.times = rec.times - time_start;

  if ~isempty(rec.picks_fb)
	rec.picks_fb(:,2) = rec.picks_fb(:,2) - time_start;
  end

  if ~isempty(rec.tt_near)
	rec.tt_near(end) = rec.tt_near(end) - time_start;
  end

  %% truncate the time range

  if isempty(time_range)
	time_range = procdata.time_range;
  end

  if ~isempty(time_range)
	ind1 = find(rec.times(:,1) >= time_range(1) & rec.times(:,1) <= time_range(2));

	rec.times = rec.times(ind1,:);
	rec.data = rec.data(ind1,:);
	data = data(ind1,:);		% filtered version of rec.data
  end

  % save data in binary Matlab format
  
  dataf = sprintf('data/f%05d.mat',rec.ffid);

  if inOctave()
  	save(dataf,'-mat7-binary','rec');
  else
	save(dataf,'rec');
  end

  % plot

  gain = 0.6;	% plot excursion

  figure(1)
  clf		% this clears the figure in case it already exists
	
  subplot(4,1,1:3)

  hold on	% this makes the different plot() commands to overlay the curves

  % draw a green circle at channel number corresponding to the position of the source  

  plot(rec.src_stn+rec.src_skid - rec.rec_stn(1) + rec.chan(1),0.0,'go')

  for i=1:length(rec.chan)
	chan = rec.chan(i);

	d = data(:,i);
	t = rec.times;

	% draw the trace 

	if ismember(chan,rec.kill)
 		plot(d/(max(abs(d))+1.e-8)*gain+chan,t,'r-')
	else
 		plot(d/(max(abs(d))+1.0e-8)*gain+chan,t,'b-')
	end
  end

  % draw the tt model in near traces by black lines

  if ~isempty(rec.tt_near)
  	plot(rec.chan(ind_near),polyval(rec.tt_near,abs(rec.offset(ind_near))),'k-')
  end

  % draw crosses at first-arrival picks  

  if ~isempty(rec.picks_fb)
  	plot(rec.chan(rec.picks_fb(:,1)),rec.picks_fb(:,2),'kx')
  end

  xlim([rec.chan(1)-1, rec.chan(end)]+1)
  ylim([rec.times(1), rec.times(end)])

  set(gca,'Ydir','reverse');
  set(gca,'XTick',rec.chan(1):2:rec.chan(end));

  ylabel("Time (ms)")
  xlabel("Channel number / Ground velocity")
  title( sprintf("Line %d; FFID %d; SRC at stn=%d skid=%1.2f m offset=%1.2f m; ts = %1.1f ms", ...
		procdata.line, ffid, rec.src_stn, rec.src_skid*rec.line(4), rec.src_offset, time_start ) )

  subplot(4,1,4)

  plot(rec.chan,rec.offset,'b-')

  grid on
  ylabel("Offset (m)")
  xlabel("Channel number")

end



