% Obtains parameters of a given FFID from the observer's log.
% ==============================================================
% The log is scanned for matching FFID number, and contents of the
% matching line are returned.
% These parameters are also duplicated in .mat file, but the log
% can be edited in case of errors.
%
% Parameters:
%	log_file - log file name to use. If empty value [] is given, the standard name
%		obtained by file_names(ffid) is used
%	ffid - FFID number to extract from the log file
%		If the trial number is not found in the log, error state is caused
%		and the program stopped
% Returns:
%	line_par - row with three numbers:
%			line number from log header for each output record
%			base station number
%			base X coordinate
%	src_no - source point number
%	src_stn - source surface station number
%	rec_stn - array (row) of two values containing receiver station numbers for the first and last receiver
%	trigger - number of trigger

function [ line_par, src_no, src_stn, rec_stn, trigger ] = params_ffid(log_file,ffid)

	if isempty(log_file)
		log_file = file_names(ffid);
	end

	fid = fopen(log_file,'r');

	if fid < 0
		error("Cannot open file %s for reading",log_file)
	end

	line_par = zeros(1,4);	% default in case no line number in the header

	s = fgets(fid);		% read first line

	while s > 0

		s = strtrim(s);		% trim spaces

		if length(s) < 1		% skip blank line
			s = fgets(fid);			
			continue
		end

		if s(1) == '%' || s(1) == '#'		% skip commentary but extract header parameters

			[ ~, s ] = strtok(s);		

			[ ind, tok ] = params_header(s);	% try recognizing a keyword

			if ind > 0
				line_par(1,ind) = str2num(tok);
			end

			s = fgets(fid);			
			continue
		end

		str = s;				% current line

		[ tok, s ] = strtok(s);			% first token

		src_no = str2num(tok);

		[ tok, s ] = strtok(s);			% second token

		if str2num(tok) == ffid		% trial number found - make outputs

			% obtain other parameters

			[ tok, s ] = strtok(s);	

			src_stn = str2num(tok);

			[ tok, s ] = strtok(s);	

			rec_stn(1) = str2num(tok);

			[ tok, s ] = strtok(s);	

			rec_stn(2) = str2num(tok);

			[ tok, s ] = strtok(s);

			trigger = str2num(tok);

			fclose(fid);
			return			% done
		end

		s = fgets(fid);		% next line
	end
	
	fclose(fid);

	% if got here, this ffid was not found in this log

	error("No FFID = %d in log file %s",ffid, log_file)
end

