% Obtains parameters of a all FFIDs from the observer's log.
% =============================================================================
% The log is scanned for non-commentary entries and columns of
% line number values and FFID parameters output as in params_ffid.m
%
% Parameters:
%	log_file - log file name to use. If empty value [] is given, the standard name
%		obtained by file_names(ffid) is used
% Returns:
%	line_par - line parameters: matrix with three numbers per row:
%			column of line numbers from log header for each output record
%			column of base station numbers
%			column of base coordinates
%	ffid - column of FFID numbers (the same lingth as line_par)
%	src_no - column of source point numbers
%	src_stn - column of source station numbers
%	rec_stn - matrix of rows containing receiver station numbers for the first and last receiver
%	trigger - column of trigger numbers

function [ line_par, ffid, src_no, src_stn, rec_stn, trigger ] = params_line(log_file)

	if isempty(log_file)
		log_file = file_names(ffid);
	end

	fid = fopen(log_file,'r');

	if fid < 0
		error("Cannot open file %s for reading",log_file)
	end

	line_par = [];
	ffid = [];
	src_no = [];
	src_stn = [];
	rec_stn = [];
	trigger = [];

	line_no = 0;		% default in case no line number in the header

	s = fgets(fid);		% read first line

	while s > 0

		s = strtrim(s);		% trim spaces

		if length(s) < 1		% skip blank line
			s = fgets(fid);			
			continue
		end

		if s(1) == '%' || s(1) == '#'		% skip commentary but extract line number

			[ ~, s ] = strtok(s);		
			[ tok, s ] = strtok(s);			% second token

			if strcmpi(tok,'line')
				[ tok, s ] = strtok(s);		% line with line number 

				if strfind(tok,':') || strfind(tok,'=')		% skip ':' or '='
					[ tok, s ] = strtok(s);
				end

				line_no = str2num(tok);
			end

			s = fgets(fid);			
			continue
		end

		str = s;				% current line

		[ tok, s ] = strtok(s);			% first token

		line_par = [ line_par; line_no ];	% output line number

		src_no = [ src_no; str2num(tok) ];

		[ tok, s ] = strtok(s);			% second token

		ffid = [ ffid; str2num(tok) ];		% output FFID number

		% obtain other parameters

		[ tok, s ] = strtok(s);	

		src_stn = [ src_stn; str2num(tok) ];

		[ tok, s ] = strtok(s);	

		r(1) = str2num(tok);

		[ tok, s ] = strtok(s);	

		r(2) = str2num(tok);

		rec_stn = [ rec_stn; r ];

		[ tok, s ] = strtok(s);

		trigger = [ trigger; str2num(tok) ];

		s = fgets(fid);		% next line
	end
	
	fclose(fid);
end

