%% Extracts RGIM seismograph record by FFID number
%% ================================================================================

%% The record is extracted in the form of structure 'rec' below,
%% containing the following fields:
%%	rec.ffid - FFID number in this record
%%	rec.source - source station number from data file 
%%		(set during acquisition or edited later)
%%	rec.src_stn - source station number from data file 
%%	rec.chan - array (row) of channel numbers
%%	rec.rec_stn - receiver station numbers corresponding to each chan 
%%		In the initial data, these station numbers equal 'chan' values
%%	rec.offset - array of source-receiver offsets (distances) 
%%		In raw data, this is equal (rec_stn - src_stn)
%%	rec.times - column of time sampling
%%	rec.data - matrix with rows corresponding to times and 
%%		 columns corresponding to recorded channels
%%	rec.kill - array of kill flags - numbers of channels to 
%%		skip in most processing. In the original record from the
%%		seismograph, this array is empty
%%	rec.trigger - trigger count number
%%	rec.trigger_time - array of trigger times from each of the 12 boards
%% ================================================================================
%% Parameters:
%%	ffid0 - requested ffid number
%%	dataset - optional string parameter. If dataset='raw', raw field records
%%		from directory 'mat/' are loaded. If dataset='edited' or any other,
%%		edited data from directory 'data/' are loaded.
%%		If this parameter is not given, 'edited' is assumed.
%% Returns:
%%	rec - structure containing the record (empty if no such record exists)
%%	spec - structure containing the record spectra (returned from loading
%%		edited and processed data)
%%	dispsemb - structure containing dispersion results
%%	matfile - name of .mat file from which the data were loaded
%%
%% Usage examples:
%%	rec1 = record_ffid(150)

function [ rec, spec, dispsemb, matfile ] = record_ffid(ffid0,dataset)

global procdata

  if nargin < 1
	error("FFID not given")
  end

  if ismember(ffid0,procdata.drop_ffid)		% skip unwanted ffids
	rec=[];
	return
  end

  if nargin < 2
	dataset = 'edited';
  end

  [ logfile, matfile ] = file_names(ffid0,dataset);

  if exist(matfile)

	if strcmpi(dataset,'raw')
		load(matfile)

		spec=[];	% no spectra for raw data
	else
		% check whether we are in Octave or Matlab

		if inOctave()
			load(matfile,'-mat')
		else
			load(matfile)		% this loads the entire structure 'rec'
		end

		if ~exist('spec')
			spec=[];
		end

		if ~exist('dispsemb')
			dispsemb=[];
		end

		return
	end

	% load shot params from log file 
	% (ignore 'trigger' because it should certainly be accurate in the .mat file)

	[ rec.line, rec.source, rec.src_stn, rstn ] = params_ffid(logfile,ffid);

	rec.ffid = ffid;

	rec.chan = chan;
	rec.rec_stn = rstn(1):1:rstn(2);

	rec.offset = rec.rec_stn - rec.src_stn;
	rec.data = data;
	rec.times = transpose(0:(size(data,1)-1))*timing(2) + timing(1);

  	rec.dt = timing(2);		% sampling interval

	rec.trigger = trigger;
	rec.trigger_time = trigger_time;
	rec.kill = [];
  else
	rec = [];
  end

end

