%% Dispersion analysis for an arbitrary record and spectra pair
%% =====================================================================
%% 
%%	General processing parameters need to be provided in structure 'procdata':
%%
%%	Creates a plot in figure(31)
%%
%% Parameters:
%%	ffid - FFID number for which to create phase-velocity spectra
%% Returns:
%%	dispsemb - structure containing dispersion spectra

%% Usage examples:
%%	measure_dispersion(150)

function dispsemb = semblance_vphase(rec, spec, procdata, offset_range)

  dispsemb.phase_vel_expected = procdata.phase_vel_expected;
  dispsemb.phase_vel = procdata.phase_vel;
  dispsemb.offset_range = procdata.offset_range;
  dispsemb.mode_separ = procdata.mode_separ;
  dispsemb.freq_min = procdata.freq_min;

  if nargin < 4
	offset_range = procdata.offset_range(1,:);
  end

  spacing = abs(rec.rec_x(end) - rec.rec_x(1))/abs(rec.chan(end) - rec.chan(1)); % nominal spacing between receivers
      
  dispsemb.freq_range = [ 0, 0.5*dispsemb.phase_vel_expected/spacing ];

  dispsemb.slwns = linspace(1.0/dispsemb.phase_vel(2),1.0/dispsemb.phase_vel(1),200);	% linear grid of slownesses

%% This needs to be revised if used here:
%% test: replace the records of all receivers with the first one shifted in time
%% proportionally to distance.
%% You should see in the diagram a dispsemb.semblance high at constsant velocity given by the
%% third parameter in this call: 

%  rec = test_moveout(rec,dist,0.25);

%% form column of output frequencies (from 0 to Nyquist, within dispsemb.freq_range )

  dispsemb.f = transpose(0:(spec.fN-1))*spec.fi;

  skip_f = find(dispsemb.f < dispsemb.freq_range(1) | dispsemb.f > dispsemb.freq_range(2) ); % range of frequencies to skip

  % select records as: 	1) not killed, 
  %			2) within dispsemb.offset_range for fundamental mode
  %			3) within procdata.chan_range if specified
  %			4) within procdata.dist_range if specified
  % and truncate the spectra to the requested frequency range 

  aoffs = abs(rec.offset);
  maxoffs = max(aoffs);

  sel = ~ismember(rec.chan,rec.kill) & aoffs >= offset_range(1,1) & aoffs <= offset_range(1,2);

  if size(procdata.chan_range,1) >= 1

	sel1 = rec.chan >= procdata.chan_range(1,1) & rec.chan <= procdata.chan_range(1,2);

  	for i = 2:size(procdata.chan_range,1)	
		sel1 = sel1 | rec.chan >= procdata.chan_range(i,1) & rec.chan <= procdata.chan_range(i,2);
	end

	sel = sel & sel1;
  end

  if size(procdata.dist_range,1) >= 1

	sel1 = aoffs >= procdata.dist_range(1,1)*maxoffs & aoffs <= procdata.dist_range(1,2)*maxoffs;

  	for i = 2:size(procdata.dist_range,1)	
		sel1 = sel1 | aoffs >= procdata.dist_range(i,1)*maxoffs & ...
				aoffs <= procdata.dist_range(i,2)*maxoffs;
	end

	sel = sel & sel1;
  end

  chind = find(sel);			% indices of records to use
  dispsemb.chans = rec.chan(chind);		% output list of channels

  spec1 = spec.data(:,chind);

  spec1((spec.fN+1):end,:) = [];
  spec1(skip_f,:) = [];
  dispsemb.f(skip_f,:) = [];

%sf = size(dispsemb.f)		% test printouts
%ss = size(spec1)
%so = size(rec.offset)

%% 3) normalize the spectral amplitudes (keep only the phase factor)

  spec1 = spec1 ./ (abs(spec1)+1.0e-16);

%% 4) for each trial slowness, apply a phase shift to align each record with the first one,
%%	 sum them, and take abs()

  two_pi_i_f_d = transpose(1:length(dispsemb.f))*spec.fi*(2i)*pi*abs(rec.offset(chind)); 
						% matrix of factors in complex exponents

  dispsemb.semblance = zeros(size(spec1,1),length(dispsemb.slwns));	% initialize a zero matrix for outputs

  for i=1:length(dispsemb.slwns)
	dispsemb.semblance(:,i) = abs(mean(spec1.*exp(two_pi_i_f_d*dispsemb.slwns(i)),2));
  end

%ds = dispsemb.semblance

%% 5) locate peaks of slownesses and determine their phase velocities

  % calculate a curve to indicate aliasing in p

  dispsemb.p_alias = ones(length(dispsemb.f),1)*max(dispsemb.slwns);	% constant equal max slowness in plot
  p1 = 0.5*dispsemb.f.^(-1)/spacing;

  ind = find(p1 < max(dispsemb.slwns));	% indices of the part within slowness plot range
  dispsemb.p_alias(ind) = p1(ind);

  % find semblance peaks lying below dispsemb.p_alias

  dispsemb.semb_peak = [];		% these will be peaks close to the fundamental mode 
  dispsemb.semb_peak1 = [];		% these will be other peaks

  ind1 = find(dispsemb.f >= dispsemb.freq_min & dispsemb.f <= dispsemb.mode_separ(1));	
						% picking frequencies below the separator
  ind2 = find(dispsemb.f >= dispsemb.freq_min & dispsemb.f > dispsemb.mode_separ(1));	
						% frequencies above the separator

  % locate peaks below separator frequency

  for i=1:length(ind1)

	i1 = ind1(i);

	ind = find(dispsemb.slwns < dispsemb.p_alias(i1));  % slownesses below the aliasing curve

	if ~isempty(ind)

  		[ s, i2 ] = max(dispsemb.semblance(i1,ind));

		dispsemb.semb_peak = [ dispsemb.semb_peak; dispsemb.f(i1), dispsemb.slwns(ind(i2)), s ];
	end
  end

  % locate peaks above separator frequency:   slowness(f) = s(2) - s(3)/(frequency - s(1))

  separ = dispsemb.mode_separ(2) - dispsemb.mode_separ(3)*(dispsemb.f(ind2) - dispsemb.mode_separ(1)).^(-1);

  dispsemb.p_separ = zeros(size(dispsemb.f));		% array for plotting the separator
  dispsemb.p_separ(ind2) = separ;

  for i=1:length(ind2)

	i1 = ind2(i);

	ind = find(dispsemb.slwns < dispsemb.p_alias(i1) & ...
			dispsemb.slwns > separ(i) );	% slownesses below the aliasing curve
							% and above separator
	if ~isempty(ind)				
  		[ s, i2 ] = max(dispsemb.semblance(i1,ind));

		dispsemb.semb_peak = [ dispsemb.semb_peak; dispsemb.f(i1), dispsemb.slwns(ind(i2)), s ];	
	end

	ind = find(dispsemb.slwns < dispsemb.p_alias(i1) & ...
			dispsemb.slwns <= separ(i) );	% slownesses below the aliasing curve
							% and below separator
	if ~isempty(ind)
  		[ s, i2 ] = max(dispsemb.semblance(i1,ind));	

		dispsemb.semb_peak1 = [ dispsemb.semb_peak1; dispsemb.f(i1), dispsemb.slwns(ind(i2)), s ];
  						% these are counted as dispsemb.semb_peak1
	end
  end

%dsp = dispsemb.semb_peak
%dsp1 = dispsemb.semb_peak1

end

